(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.headTo = factory());
}(this, (function () { 'use strict';

// Robert Penner's easeInOutQuad

// find the rest of his easing functions here: http://robertpenner.com/easing/
// find them exported for ES6 consumption here: https://github.com/jaxgeller/ez.js

var easeInOutQuad = function(t, b, c, d) {
  t /= d / 2;
  if (t < 1) { return c / 2 * t * t + b }
  t--;
  return -c / 2 * (t * (t - 2) - 1) + b
};

var jumper = function () {
  // private variable cache
  // no variables are created during a jump, preventing memory leaks

  var container; // scrollable element which contains target element (node)

  var element; // element to scroll to                   (node)

  var start; // where scroll starts                    (px)
  var stop; // where scroll stops                     (px)

  var offset; // adjustment from the stop position      (px)
  var easing; // easing function                        (function)
  var a11y; // accessibility support flag             (boolean)

  var distance; // distance of scroll                     (px)
  var duration; // scroll duration                        (ms)

  var timeStart; // time scroll started                    (ms)
  var timeElapsed; // time spent scrolling thus far          (ms)

  var next; // next scroll position                   (px)

  var callback; // to call when done scrolling            (function)

  // scroll position helper

  function location() {
    return (
      container.scrollY || container.pageYOffset || container.scrollTop || 0
    )
  }

  // element offset helper

  function top(element) {
    var elementTop = element.getBoundingClientRect().top;
    var containerTop = container.getBoundingClientRect
      ? container.getBoundingClientRect().top
      : 0;

    return elementTop - containerTop + start
  }

  // scrollTo helper

  function scrollTo(top) {
    if (container.scrollTo) {
      container.scrollTo(0, top); // window
    } else {
      container.scrollTop = top; // custom container
    }
  }

  // rAF loop helper

  function loop(timeCurrent) {
    // store time scroll started, if not started already
    if (!timeStart) {
      timeStart = timeCurrent;
    }

    // determine time spent scrolling so far
    timeElapsed = timeCurrent - timeStart;

    // calculate next scroll position
    next = easing(timeElapsed, start, distance, duration);

    // scroll to it
    scrollTo(next);

    // check progress
    if (timeElapsed < duration) {
      window.requestAnimationFrame(loop); // continue scroll loop
    } else {
      done(); // scrolling is done
    }
  }

  // scroll finished helper

  function done() {
    // account for rAF time rounding inaccuracies
    scrollTo(start + distance);

    // if scrolling to an element, and accessibility is enabled
    if (element && a11y) {
      // add tabindex indicating programmatic focus
      element.setAttribute('tabindex', '-1');

      // focus the element
      element.focus();
    }

    // if it exists, fire the callback
    if (typeof callback === 'function') {
      callback();
    }

    // reset time for next jump
    timeStart = false;
  }

  // API

  function jump(target, options) {
    if ( options === void 0 ) options = {};

    // resolve options, or use defaults
    duration = options.duration || 1000;
    offset = options.offset || 0;
    callback = options.callback; // "undefined" is a suitable default, and won't be called
    easing = options.easing || easeInOutQuad;
    a11y = options.a11y || false;

    // resolve container
    switch (typeof options.container) {
      case 'object':
        // we assume container is an HTML element (Node)
        container = options.container;
        break

      case 'string':
        container = document.querySelector(options.container);
        break

      default:
        container = window;
    }

    // cache starting position
    start = location();

    // resolve target
    switch (typeof target) { // eslint-disable-line default-case
      // scroll from current position
      case 'number':
        element = undefined; // no element to scroll to
        a11y = false; // make sure accessibility is off
        stop = start + target;
        break
      // scroll to element (node)
      // bounding rect is relative to the viewport
      case 'object':
        element = target;
        stop = top(element);
        break
      // scroll to element (selector)
      // bounding rect is relative to the viewport
      case 'string':
        element = document.querySelector(target);
        stop = top(element);
        break
    }

    // resolve scroll distance, accounting for offset
    distance = stop - start + offset;

    // resolve duration
    switch (typeof options.duration) { // eslint-disable-line default-case
      // number in ms
      case 'number':
        duration = options.duration;
        break
      // function passed the distance of the scroll
      case 'function':
        duration = options.duration(distance);
        break
    }

    // start the loop
    window.requestAnimationFrame(loop);
  }

  // expose only the jump method
  return jump
};

// export singleton
var index = jumper();

return index;

})));
